/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * Translation by Torsten Krueger (ToK).
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Table.h"
#include "Text.h"


// ------------------------------------------------------------------
// Constants
// ------------------------------------------------------------------
const Int8 TABLE_ROWS   = MAX_TEAMS;
const Int8 VISIBLE_ROWS = 11;

// ------------------------------------------------------------------
// Globals declared elsewhere
// ------------------------------------------------------------------
extern AppState GState;

// ------------------------------------------------------------------
// Global variables
// ------------------------------------------------------------------
MemHandle GRoundFieldHandle;


// ------------------------------------------------------------------
// Initializes the form.
// ------------------------------------------------------------------
void InitTableForm() {
  MemPtr       resPtr;
  ScrollBarPtr tableScl;
  League       league = LeagueGetCurrent();

  InitTop(MODE_TABLE);

  tableScl = (ScrollBarPtr)UIGetObjectPtr(scrollID_table);
  SclSetScrollBar(tableScl, GState.Table.scrollPos, 0, max(league.teamCount-VISIBLE_ROWS, 0), VISIBLE_ROWS);

  UISelectDropDown(popListID_matches, popTrigID_matches, GState.Table.matches);
  UISelectDropDown(popListID_rounds, popTrigID_rounds, GState.Table.rounds);

  // init the round count field
  GRoundFieldHandle = MemHandleNew(3);
  resPtr = MemHandleLock(GRoundFieldHandle);
  StrIToA(resPtr, GState.Table.roundCount);
  MemHandleUnlock(GRoundFieldHandle);
  FldSetTextHandle(UIGetObjectPtr(fieldID_roundCount), GRoundFieldHandle);
}


// ------------------------------------------------------------------
// Frees resources allocated for this form.
// ------------------------------------------------------------------
void FreeTableForm() {
  FldSetTextHandle(UIGetObjectPtr(fieldID_roundCount), NULL);
  MemHandleFree(GRoundFieldHandle);
}


// ------------------------------------------------------------------
// Updates the controls and data on the form.
// ------------------------------------------------------------------
void UpdateTableForm() {
  FormPtr form = FrmGetActiveForm();

  // show or hide round count field and update button
  if(GState.Table.rounds == 0) {
    FrmHideObject(form, FrmGetObjectIndex(form, fieldID_roundCount));
    FrmHideObject(form, FrmGetObjectIndex(form, btnID_roundUpdate));
  }
  else {
    FrmShowObject(form, FrmGetObjectIndex(form, fieldID_roundCount));
    FrmShowObject(form, FrmGetObjectIndex(form, btnID_roundUpdate));
  }

  // set values in drop-downs (because of plamos5 bug?)
  UISelectDropDown(popListID_matches, popTrigID_matches, GState.Table.matches);
  UISelectDropDown(popListID_rounds, popTrigID_rounds, GState.Table.rounds);
}


// ------------------------------------------------------------------
// Draws a table row on the screen.
// ------------------------------------------------------------------
void DrawTableRow(League* league, TableTeam* team, Int8 mode) {
  Char  buf[20];
  Char* tempPtr;
  Int16 y = 28 + ((team->position - GState.Table.scrollPos) * 11);

  // position
  StrIToA(buf, team->position);
  StrCat(buf, ".");
  UIDrawCharsRight(buf, 14, y);

  // team name
  tempPtr = LeagueGetTeam(league, team->teamId);
  StrCopy(buf, tempPtr);
  MemPtrUnlock(tempPtr);
  UIDrawCharsLeft(buf, 15, y);

  // played matches
  StrIToA(buf, team->played);
  UIDrawCharsRight(buf, 83, y);

  switch(mode) {
    case 0: // show goals for and against and goal diff
      StrIToA(buf, team->gfor);
      StrCat(buf, TEXT_TABLE_GOAL_SEP);
      UIDrawCharsRight(buf, 104, y);
      StrIToA(buf, team->against);
      UIDrawCharsLeft(buf, 104, y);
      StrPrintF(buf, "%+d", team->goalDiff);
      UIDrawCharsRight(buf, 134, y); //136 /*ToK: changed from 152*/
      StrIToA(buf, team->points);
      UIDrawCharsRight(buf, 152, y); //152 /*ToK: changed from 131*/
      break;

    case 1: // show wins, draws, and losses
      StrIToA(buf, team->won);
      UIDrawCharsRight(buf, 102, y);
      StrIToA(buf, team->drawn);
      UIDrawCharsRight(buf, 117, y);
      StrIToA(buf, team->lost);
      UIDrawCharsRight(buf, 132, y);
      StrIToA(buf, team->points);
      UIDrawCharsRight(buf, 152, y);
      break;

  }

}


// ------------------------------------------------------------------
// Draws the form.
// ------------------------------------------------------------------
void DrawTableForm() {
  Int16      i, y;
  League     league      = LeagueGetCurrent();
  Boolean    includeHome = (GState.Table.matches == MATCHES_ALL || GState.Table.matches == MATCHES_HOME);
  Boolean    includeAway = (GState.Table.matches == MATCHES_ALL || GState.Table.matches == MATCHES_AWAY);
  Boolean    forward     = (GState.Table.rounds == 0 || GState.Table.rounds == 1);
  Int16      maxMatches  = (GState.Table.rounds == 0) ? -1 : GState.Table.roundCount;
  Table      table       = TableCreate(&league, includeHome, includeAway, forward, maxMatches);
  TableTeam* team;

  // erase the draw window (due to bug earlier OS versions)
  RectangleType eraseRect = { { 0, 39 } , { 152, 121 } };
  WinEraseRectangle(&eraseRect, 0);

  // draw the table rows
  for (i = 0; i < league.teamCount; i++) {
    team = &table.teams[i];
    // we check the scrollbar's position to see if we should draw the team
    if (team->position > GState.Table.scrollPos &&
        team->position <= (VISIBLE_ROWS + GState.Table.scrollPos)) {
      DrawTableRow(&league, team, GState.Table.view);
    }
  }

  // draw lines in the table
  for (i = 0; i < 4; i++) {
    y = 49 + (league.line[i]-GState.Table.scrollPos)*11;
    if (y >= 49) {
      WinDrawGrayLine(3, y, 151, y);
    }
  }

}


// ------------------------------------------------------------------
// Scrolls the table.
// ------------------------------------------------------------------
void ScrollTable(Int8 delta) {
  ScrollBarPtr tableScl;
  League league    = LeagueGetCurrent();
  Int16  teamCount = league.teamCount;

  GState.Table.scrollPos += delta;
  if(GState.Table.scrollPos > teamCount-VISIBLE_ROWS) {
    // this might actually set scrollPos to less than zero but that
    // will be corrected below
    GState.Table.scrollPos = teamCount-VISIBLE_ROWS;
  }
  if(GState.Table.scrollPos < 0) {
    GState.Table.scrollPos = 0;
  }
  // update the scrollbar
  tableScl = (ScrollBarPtr)UIGetObjectPtr(scrollID_table);
  SclSetScrollBar(tableScl, GState.Table.scrollPos, 0, max(teamCount-VISIBLE_ROWS, 0), VISIBLE_ROWS);
}


// ------------------------------------------------------------------
// Eventhandler for the form.
// ------------------------------------------------------------------
Boolean TableFormEvtHandler(EventPtr event) {
  Boolean handled      = false;
  Boolean updateScreen = false;

  switch (event->eType) {

   // form is opened
    case frmOpenEvent:
      InitTableForm();
      updateScreen = true;
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeTableForm();
      break;

   // form need to be updated
    case frmUpdateEvent:
      updateScreen = true;
      handled = true;
      break;

   // scrollbar is scrolled
    case sclRepeatEvent:
      GState.Table.scrollPos  = event->data.sclRepeat.newValue;
      GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
      // because of a bug with redrawing scroll bars we submit a frmUpdateEvent
      // to re-draw the form instead of re-drawing it directly.
      FrmUpdateForm(formID_table, 0);
      break;

   // one of the scrollbuttons was pressed
    case keyDownEvent:
      switch(event->data.keyDown.chr) {
        case pageUpChr :
          ScrollTable(-VISIBLE_ROWS);
          GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
          handled = true;
          updateScreen = true;
          break;
        case pageDownChr:
          ScrollTable(VISIBLE_ROWS);
          GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
          handled = true;
          updateScreen = true;
          break;
      }
      break;

   // a button was tapped
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {
        case btnID_matches:
          GState.mode = MODE_MATCHES;
          FrmGotoForm(formID_matches);
          handled = true;
          break;
        case btnID_team:
          GState.mode = MODE_TEAM;
          FrmGotoForm(formID_team);
          handled = true;
          break;
        case btnID_league:
          GState.mode = MODE_LEAGUE;
          FrmGotoForm(formID_league);
          handled = true;
          break;

        case btnID_roundUpdate:
          GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
          handled = true;
          updateScreen = true;
          break;
        case selectTrigID_table_view:
          GState.Table.view = (GState.Table.view+1) % 2;
          updateScreen = true;
      }
      break;

   // drop-down selected
    case popSelectEvent:
      switch(event->data.popSelect.controlID) {
        case popTrigID_league:
          // change order of leagues
          if (event->data.popSelect.selection == GState.leagueNameCount) {
            FrmPopupForm(formID_leagueOrder);
          }
          // show another league
          else if (event->data.popSelect.selection != GState.currentLeagueNameIndex) {
            ShowLeague(event->data.popSelect.selection, 0, MODE_TABLE, true, true);
          }
          break;
        case popTrigID_season:
          // show another league
          if (event->data.popSelect.selection != GState.currentSeasonIndex) {
            ShowLeague(GState.currentLeagueNameIndex, event->data.popSelect.selection, MODE_TABLE, true, false);
          }
          break;
        case popTrigID_matches:
          GState.Table.matches = event->data.popSelect.selection;
          updateScreen = true;
          break;
        case popTrigID_rounds:
          GState.Table.rounds = event->data.popSelect.selection;
          updateScreen = true;
          break;
      }
      GState.Table.roundCount = UIGetFieldIntValue(fieldID_roundCount);
      break;


    default:

  }

 // redraw the screen if needed
  if(updateScreen) {
    UpdateTableForm();
    FrmDrawForm(FrmGetActiveForm());
    DrawTableForm();
  }

  return handled;
}
