/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.4.0 November 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.mbovin.palmos;

import java.util.*;

/**
 * This class represents a PalmOS database.
 */
public class Database
{
    protected String mName;
    protected int    mAttributes;
    protected int    mVersion;
    protected int    mCreationDate = 1;
    protected int    mBackupDate = 1;
    protected int    mModificationDate = 1;
    protected int    mModificationNumber;
    protected int    mAppInfo;
    protected int    mSortInfo;
    protected String mDBType = "Data";
    protected String mCreator;
    protected int    mUniqueIdSeed;
    protected int    mNextRecordList;
    protected List   mRecords = new ArrayList();


    /** Constructor. */
    public Database() {}

    /** Constructor. */
    public Database(String name, String creator)
    {
        mName    = name;
        mCreator = creator;
    }


    /** Adds a record to this database. */
    public void addRecord(Record rec)
    {
        mRecords.add(rec);
    }


    /** Returns the number of records in the database. */
    public int getRecordCount()
    {
        return mRecords.size();
    }

    /** Returns an iterator for the records. */
    public Iterator getRecordIterator()
    {
        return mRecords.iterator();
    }

    /** Returns the record at the specified index. */
    public Record getRecordAt(int recordIndex)
    {
        return (Record)mRecords.get(recordIndex);
    }


    /**
     * Getter for property Attributes.
     * @return Value of property Attributes.
     */
    public int getAttributes()
    {
        return mAttributes;
    }

    /**
     * Setter for property Attributes.
     * @param Attributes New value of property Attributes.
     */
    public void setAttributes(int attributes)
    {
        mAttributes = attributes;
    }

    /**
     * Getter for property Name.
     * @return Value of property Name.
     */
    public java.lang.String getName()
    {
        return mName;
    }

    /**
     * Setter for property Name.
     * @param name New value of property Name.
     */
    public void setName(java.lang.String name)
    {
        mName = name;
    }

    /**
     * Getter for property Creator.
     * @return Value of property Creator.
     */
    public java.lang.String getCreator()
    {
        return mCreator;
    }

    /**
     * Setter for property Creator.
     * @param creator New value of property Creator.
     */
    public void setCreator(java.lang.String creator)
    {
        mCreator = creator;
    }

    /**
     * Getter for property DBType.
     * @return Value of property DBType.
     */
    public java.lang.String getDBType()
    {
        return mDBType;
    }

    /**
     * Setter for property DBType.
     * @param dbType New value of property DBType.
     */
    public void setDBType(java.lang.String dbType)
    {
        mDBType = dbType;
    }

}
