/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.4.0 November 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.mbovin.palmos;

import java.io.*;

/**
 * This class represents a record in a PalmOS database.
 */
public class Record
{
    protected int  mLocalId;
    protected byte mAttributes;
    protected int  mUniqueId;
    protected ByteArrayOutputStream mByteStream;
    protected DataOutputStream mDataStream;


    /** Constructor. */
    public Record()
    {
        this(0, (byte)0);
    }

    /** Constructor. */
    public Record(int localId, byte attributes)
    {
        mLocalId    = localId;
        mAttributes = attributes;
        mByteStream = new ByteArrayOutputStream();
        mDataStream = new DataOutputStream(mByteStream);
    }


    /** Returns the localId of this record */
    public int getLocalId()
    {
        return mLocalId;
    }

    /** Sets the localId of this record */
    public void setLocalId(int localId)
    {
        mLocalId = localId;
    }

    /** Returns the attributes of this record */
    public byte getAttributes()
    {
        return mAttributes;
    }

    /** Sets the localId of this record */
    public void setAttributes(byte attributes)
    {
        mAttributes = attributes;
    }


    /** Adds a byte to the record. */
    public void addByte(int b)
    {
        try { mDataStream.writeByte(b); }
        catch(java.io.IOException e) {}
    }

    /** Adds a short to the record. */
    public void addShort(int s)
    {
        try { mDataStream.writeShort(s); }
        catch(java.io.IOException e) {}
    }

    /** Adds an int to the record. */
    public void addInt(int i)
    {
        try { mDataStream.writeInt(i); }
        catch(java.io.IOException e) {}
    }

    /** Adds a string to the record. */
    public void addString(String s)
    {
        try { mDataStream.writeBytes(s); }
        catch(java.io.IOException e) {}
    }

    /**
     * Adds a string of a specified size to the record. If the string is longer
     * it's truncated and if it's shorter it's padded.
     */
    public void addString(String s, int size)
    {
        StringBuffer buf = new StringBuffer(s);
        buf.setLength(size);
        addString(buf.toString());
    }


    /**
     * Reads bytes from a stream and adds them to the record data.
     * @param in the stream to read from
     * @param length the number of bytes to read
     */
    public void readData(DataInputStream in, int length)
        throws java.io.IOException
    {
        byte[] bytes = new byte[length];
        in.read(bytes);
        mDataStream.write(bytes);
    }


    /** Returns the size (in bytes) of this record. */
    public int getSize()
    {
        return mByteStream.size();
    }

    /** Returns the data of this record. */
    public byte[] getBytes()
    {
        return mByteStream.toByteArray();
    }

}
